@@ -18,6 +18,8 @@ Dataset <- R6::R6Class(
 
       }
 
+      return(invisible(NULL))
+
     },
 
     # Initialize will create a starter dataset
@@ -85,6 +87,8 @@ Dataset <- R6::R6Class(
       private$info <- info
       private$version <- 0L
 
+      return(invisible(NULL))
+
     },
 
     create_valid = function(data,
@@ -325,16 +329,18 @@ Dataset <- R6::R6Class(
         num_col <- 0L
 
         # Get numeric data and numeric features
-        c(
-          lgb.call(
-            fun_name = "LGBM_DatasetGetNumData_R"
-            , ret = num_row
-            , private$handle
-          ),
-          lgb.call(
-            fun_name = "LGBM_DatasetGetNumFeature_R"
-            , ret = num_col
-            , private$handle
+        return(
+          c(
+            lgb.call(
+              fun_name = "LGBM_DatasetGetNumData_R"
+              , ret = num_row
+              , private$handle
+            ),
+            lgb.call(
+              fun_name = "LGBM_DatasetGetNumFeature_R"
+              , ret = num_col
+              , private$handle
+            )
           )
         )
 
@@ -342,7 +348,7 @@ Dataset <- R6::R6Class(
 
         # Check if dgCMatrix (sparse matrix column compressed)
         # NOTE: requires Matrix package
-        dim(private$raw_data)
+        return(dim(private$raw_data))
 
       } else {
 
@@ -368,12 +374,12 @@ Dataset <- R6::R6Class(
             , private$handle
         )
         private$colnames <- as.character(base::strsplit(cnames, "\t")[[1L]])
-        private$colnames
+        return(private$colnames)
 
       } else if (is.matrix(private$raw_data) || methods::is(private$raw_data, "dgCMatrix")) {
 
         # Check if dgCMatrix (sparse matrix column compressed)
-        colnames(private$raw_data)
+        return(colnames(private$raw_data))
 
       } else {
 
@@ -470,7 +476,7 @@ Dataset <- R6::R6Class(
         }
       }
 
-      private$info[[name]]
+      return(private$info[[name]])
 
     },
 
@@ -522,17 +528,19 @@ Dataset <- R6::R6Class(
     slice = function(idxset, ...) {
 
       # Perform slicing
-      Dataset$new(
-        data = NULL
-        , params = private$params
-        , reference = self
-        , colnames = private$colnames
-        , categorical_feature = private$categorical_feature
-        , predictor = private$predictor
-        , free_raw_data = private$free_raw_data
-        , used_indices = sort(idxset, decreasing = FALSE)
-        , info = NULL
-        , ...
+      return(
+        Dataset$new(
+          data = NULL
+          , params = private$params
+          , reference = self
+          , colnames = private$colnames
+          , categorical_feature = private$categorical_feature
+          , predictor = private$predictor
+          , free_raw_data = private$free_raw_data
+          , used_indices = sort(idxset, decreasing = FALSE)
+          , info = NULL
+          , ...
+        )
       )
 
     },
@@ -679,7 +687,7 @@ Dataset <- R6::R6Class(
       if (lgb.is.null.handle(x = private$handle)) {
         self$construct()
       }
-      private$handle
+      return(private$handle)
 
     },
 
@@ -753,18 +761,20 @@ lgb.Dataset <- function(data,
                         ...) {
 
   # Create new dataset
-  invisible(Dataset$new(
-    data = data
-    , params = params
-    , reference = reference
-    , colnames = colnames
-    , categorical_feature = categorical_feature
-    , predictor = NULL
-    , free_raw_data = free_raw_data
-    , used_indices = NULL
-    , info = info
-    , ...
-  ))
+  return(
+    invisible(Dataset$new(
+      data = data
+      , params = params
+      , reference = reference
+      , colnames = colnames
+      , categorical_feature = categorical_feature
+      , predictor = NULL
+      , free_raw_data = free_raw_data
+      , used_indices = NULL
+      , info = info
+      , ...
+    ))
+  )
 
 }
 
@@ -796,7 +806,7 @@ lgb.Dataset.create.valid <- function(dataset, data, info = list(), ...) {
   }
 
   # Create validation dataset
-  invisible(dataset$create_valid(data = data, info = info, ...))
+  return(invisible(dataset$create_valid(data = data, info = info, ...)))
 
 }
 
@@ -822,7 +832,7 @@ lgb.Dataset.construct <- function(dataset) {
   }
 
   # Construct the dataset
-  invisible(dataset$construct())
+  return(invisible(dataset$construct()))
 
 }
 
@@ -856,7 +866,7 @@ dim.lgb.Dataset <- function(x, ...) {
     stop("dim.lgb.Dataset: input data should be an lgb.Dataset object")
   }
 
-  x$dim()
+  return(x$dim())
 
 }
 
@@ -893,12 +903,11 @@ dimnames.lgb.Dataset <- function(x) {
   }
 
   # Return dimension names
-  list(NULL, x$get_colnames())
+  return(list(NULL, x$get_colnames()))
 
 }
 
 #' @rdname dimnames.lgb.Dataset
-#' @return A list with the dimension names of the dataset
 #' @export
 `dimnames<-.lgb.Dataset` <- function(x, value) {
 
@@ -932,7 +941,7 @@ dimnames.lgb.Dataset <- function(x) {
 
   # Set column names properly, and return
   x$set_colnames(colnames = value[[2L]])
-  x
+  return(x)
 
 }
 
@@ -960,7 +969,6 @@ slice <- function(dataset, ...) {
 }
 
 #' @rdname slice
-#' @return constructed sub dataset
 #' @export
 slice.lgb.Dataset <- function(dataset, idxset, ...) {
 
@@ -970,7 +978,7 @@ slice.lgb.Dataset <- function(dataset, idxset, ...) {
   }
 
   # Return sliced set
-  invisible(dataset$slice(idxset = idxset, ...))
+  return(invisible(dataset$slice(idxset = idxset, ...)))
 
 }
 
@@ -988,7 +996,11 @@ slice.lgb.Dataset <- function(dataset, idxset, ...) {
 #' \itemize{
 #'     \item \code{label}: label lightgbm learn from ;
 #'     \item \code{weight}: to do a weight rescale ;
-#'     \item \code{group}: group size ;
+#'     \item{\code{group}: used for learning-to-rank tasks. An integer vector describing how to
+#'         group rows together as ordered results from the same set of candidate results to be ranked.
+#'         For example, if you have a 100-document dataset with \code{group = c(10, 20, 40, 10, 10, 10)},
+#'         that means that you have 6 groups, where the first 10 records are in the first group,
+#'         records 11-30 are in the second group, etc.}
 #'     \item \code{init_score}: initial score is the base prediction lightgbm will boost from.
 #' }
 #'
@@ -1011,7 +1023,6 @@ getinfo <- function(dataset, ...) {
 }
 
 #' @rdname getinfo
-#' @return info data
 #' @export
 getinfo.lgb.Dataset <- function(dataset, name, ...) {
 
@@ -1020,7 +1031,7 @@ getinfo.lgb.Dataset <- function(dataset, name, ...) {
     stop("getinfo.lgb.Dataset: input dataset should be an lgb.Dataset object")
   }
 
-  dataset$getinfo(name = name)
+  return(dataset$getinfo(name = name))
 
 }
 
@@ -1042,8 +1053,9 @@ getinfo.lgb.Dataset <- function(dataset, name, ...) {
 #'     \item{\code{init_score}: initial score is the base prediction lightgbm will boost from}
 #'     \item{\code{group}: used for learning-to-rank tasks. An integer vector describing how to
 #'         group rows together as ordered results from the same set of candidate results to be ranked.
-#'         For example, if you have a 1000-row dataset that contains 250 4-document query results,
-#'         set this to \code{rep(4L, 250L)}}
+#'         For example, if you have a 100-document dataset with \code{group = c(10, 20, 40, 10, 10, 10)},
+#'         that means that you have 6 groups, where the first 10 records are in the first group,
+#'         records 11-30 are in the second group, etc.}
 #' }
 #'
 #' @examples
@@ -1065,7 +1077,6 @@ setinfo <- function(dataset, ...) {
 }
 
 #' @rdname setinfo
-#' @return the dataset you passed in
 #' @export
 setinfo.lgb.Dataset <- function(dataset, name, info, ...) {
 
@@ -1074,7 +1085,7 @@ setinfo.lgb.Dataset <- function(dataset, name, info, ...) {
   }
 
   # Set information
-  invisible(dataset$setinfo(name = name, info = info))
+  return(invisible(dataset$setinfo(name = name, info = info)))
 }
 
 #' @name lgb.Dataset.set.categorical
@@ -1106,7 +1117,7 @@ lgb.Dataset.set.categorical <- function(dataset, categorical_feature) {
   }
 
   # Set categoricals
-  invisible(dataset$set_categorical_feature(categorical_feature = categorical_feature))
+  return(invisible(dataset$set_categorical_feature(categorical_feature = categorical_feature)))
 
 }
 
@@ -1138,7 +1149,7 @@ lgb.Dataset.set.reference <- function(dataset, reference) {
   }
 
   # Set reference
-  invisible(dataset$set_reference(reference = reference))
+  return(invisible(dataset$set_reference(reference = reference)))
 }
 
 #' @name lgb.Dataset.save
@@ -1171,5 +1182,5 @@ lgb.Dataset.save <- function(dataset, fname) {
   }
 
   # Store binary
-  invisible(dataset$save_binary(fname = fname))
+  return(invisible(dataset$save_binary(fname = fname)))
 }
