@@ -720,10 +720,10 @@ void GPUTreeLearner::InitGPU(int platform_id, int device_id) {
     Log::Fatal("bin size %d cannot run on GPU", max_num_bin_);
   }
   if (max_num_bin_ == 65) {
-    Log::Warning("Setting max_bin to 63 is sugguested for best performance");
+    Log::Warning("Setting max_bin to 63 is suggested for best performance");
   }
   if (max_num_bin_ == 17) {
-    Log::Warning("Setting max_bin to 15 is sugguested for best performance");
+    Log::Warning("Setting max_bin to 15 is suggested for best performance");
   }
   ctx_ = boost::compute::context(dev_);
   queue_ = boost::compute::command_queue(ctx_, dev_);
@@ -734,8 +734,8 @@ void GPUTreeLearner::InitGPU(int platform_id, int device_id) {
   SetupKernelArguments();
 }
 
-Tree* GPUTreeLearner::Train(const score_t* gradients, const score_t *hessians) {
-  return SerialTreeLearner::Train(gradients, hessians);
+Tree* GPUTreeLearner::Train(const score_t* gradients, const score_t *hessians, bool is_first_tree) {
+  return SerialTreeLearner::Train(gradients, hessians, is_first_tree);
 }
 
 void GPUTreeLearner::ResetTrainingDataInner(const Dataset* train_data, bool is_constant_hessian, bool reset_multi_val_bin) {
